@@ -1,11 +1,11 @@
 #  File R/InitErgmProposal.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2003-2018 Statnet Commons
+#  Copyright 2003-2019 Statnet Commons
 #######################################################################
 #===========================================================================
 # The <InitErgmProposal> file contains the following 24 functions for
@@ -64,7 +64,7 @@ InitErgmProposal.CondDegreeMix <- function(arguments, nw) {
 InitErgmProposal.CondOutDegree <- function(arguments, nw) {
   proposal <- list(name = "CondOutDegree", inputs=NULL)
   if (!is.directed(nw)) # Really, this should never trigger, since the InitErgmConstraint function should check.
-    stop("The CondOutDegree proposal function does not work with an",
+    ergm_Init_abort("The CondOutDegree proposal function does not work with an",
           "undirected network.")
   
   proposal
@@ -73,7 +73,7 @@ InitErgmProposal.CondOutDegree <- function(arguments, nw) {
 InitErgmProposal.CondInDegree <- function(arguments, nw) {
   proposal <- list(name = "CondInDegree", inputs=NULL)
   if (!is.directed(nw)) # Really, this should never trigger, since the InitErgmConstraint function should check.
-    stop("The CondInDegree proposal function does not work with an",
+    ergm_Init_abort("The CondInDegree proposal function does not work with an",
           "undirected network.")
   proposal
 }
@@ -81,7 +81,7 @@ InitErgmProposal.CondInDegree <- function(arguments, nw) {
 InitErgmProposal.CondB1Degree <- function(arguments, nw) {
   proposal <- list(name = "CondB1Degree", inputs=NULL)
   if (!is.bipartite(nw)) # Really, this should never trigger, since the InitErgmConstraint function should check.
-    stop("The CondB1Degree proposal function does not work with a non-bipartite network.")
+    ergm_Init_abort("The CondB1Degree proposal function does not work with a non-bipartite network.")
   
   proposal
 }
@@ -89,14 +89,14 @@ InitErgmProposal.CondB1Degree <- function(arguments, nw) {
 InitErgmProposal.CondB2Degree <- function(arguments, nw) {
   proposal <- list(name = "CondB2Degree", inputs=NULL)
   if (!is.bipartite(nw)) # Really, this should never trigger, since the InitErgmConstraint function should check.
-    stop("The CondB2Degree proposal function does not work with a non-bipartite network.")
+    ergm_Init_abort("The CondB2Degree proposal function does not work with a non-bipartite network.")
   proposal
 }
 
 InitErgmProposal.CondDegreeDist <- function(arguments, nw) {
   proposal <- list(name = "CondDegreeDist", inputs=NULL)
   if (is.directed(nw)) {
-    message("Warning:  Using the 'degreedist' constraint with a directed network ",
+    ergm_Init_warn("Using the 'degreedist' constraint with a directed network ",
           "is currently perilous.  We recommend that you use 'outdegree' or ",
           "'idegrees' instead.")
   }
@@ -109,7 +109,7 @@ InitErgmProposal.CondDegreeDist <- function(arguments, nw) {
 InitErgmProposal.CondInDegreeDist <- function(arguments, nw) {
   proposal <- list(name = "CondInDegreeDist", inputs=NULL)
   if (!is.directed(nw)) {
-    message("Warning:  Using the 'idegreedist' constraint with an undirected network ",
+    ergm_Init_warn("Using the 'idegreedist' constraint with an undirected network ",
           "is currently perilous.  We recommend that you use 'degreedist' ",
           " instead.")
   }
@@ -122,7 +122,7 @@ InitErgmProposal.CondInDegreeDist <- function(arguments, nw) {
 InitErgmProposal.CondOutDegreeDist <- function(arguments, nw) {
   proposal <- list(name = "CondOutDegreeDist", inputs=NULL)
   if (!is.directed(nw)) {
-    message("Warning:  Using the 'odegreedist' constraint with an undirected network n",
+    ergm_Init_warn("Using the 'odegreedist' constraint with an undirected network n",
           "is currently perilous.  We recommend that you use 'degreedist' ",
           " instead.")
   }
@@ -155,7 +155,7 @@ InitErgmProposal.HammingTNT <- function(arguments, nw) {
 
 InitErgmProposal.randomtoggleNonObserved <- function(arguments, nw) {
   if(network.naedgecount(nw)==0){
-   stop("The passed network does not have any non-observed dyads.\n Hence constraining to the observed will hold the network fixed at this network.\n Either the network or the constraint need to be altered.")
+   ergm_Init_abort("The passed network does not have any non-observed dyads.\n Hence constraining to the observed will hold the network fixed at this network.\n Either the network or the constraint need to be altered.")
   }
   proposal <- list(name = "randomtoggleList", inputs=to_ergm_Cdouble(is.na(nw)))
   proposal
@@ -163,7 +163,7 @@ InitErgmProposal.randomtoggleNonObserved <- function(arguments, nw) {
 
 InitErgmProposal.NonObservedTNT <- function(arguments, nw) {
   if(network.naedgecount(nw)==0){
-   stop("The passed network does not have any non-observed dyads.\n Hence constraining to the observed will hold the network fixed at this network.\n Either the network or the constraint need to be altered.")
+   ergm_Init_abort("The passed network does not have any non-observed dyads.\n Hence constraining to the observed will hold the network fixed at this network.\n Either the network or the constraint need to be altered.")
   }
   proposal <- list(name = "listTNT", inputs=to_ergm_Cdouble(is.na(nw)))
   proposal
