@@ -1,11 +1,11 @@
 #  File R/InitErgmTerm.dgw_sp.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2003-2018 Statnet Commons
+#  Copyright 2003-2019 Statnet Commons
 #######################################################################
 
 #  ------------------------------------------------------------------ 
@@ -152,7 +152,7 @@ InitErgmTerm.desp<-function(nw, arglist, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code for esp; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code for esp; valid types are:",paste(type.vec, collapse=","))
   dname<-"esp"
   if(is.directed(nw)){
     conam <- paste("esp",type,sep=".")
@@ -186,14 +186,14 @@ InitErgmTerm.desp<-function(nw, arglist, ...) {
 #always used (since it is directedness-safe), and the user's input is
 #overridden.  UTP cannot be chosen otherwise, since it won't work.
 #
-InitErgmTerm.dgwesp<-function(nw, arglist, initialfit=FALSE, ...) {
+InitErgmTerm.dgwesp<-function(nw, arglist, initialfit=FALSE, gw.cutoff=30, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("decay","fixed","cutoff","type", "alpha"),
                       vartypes = c("numeric","logical","numeric","character", "numeric"),
-                      defaultvalues = list(0, FALSE, 30,"OTP", NULL),
+                      defaultvalues = list(0, FALSE, gw.cutoff,"OTP", NULL),
                       required = c(FALSE, FALSE, FALSE, FALSE, FALSE))
   if(!is.null(a$alpha)){
-    stop("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".", call.=FALSE)
+    ergm_Init_abort("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".")
   }
   
   decay<-a$decay;fixed<-a$fixed
@@ -202,10 +202,10 @@ InitErgmTerm.dgwesp<-function(nw, arglist, initialfit=FALSE, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code for gwesp; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code for gwesp; valid types are:",paste(type.vec, collapse=","))
   dname<-"desp"
   if(!is.directed(nw)){  
-    stop("Use the gwesp term for undirected networks.")
+    ergm_Init_abort("Use the gwesp term for undirected networks.")
   }else{
     typecode<-which(type==type.vec)
     basenam<-paste("gwesp",type,sep=".")
@@ -267,7 +267,7 @@ InitErgmTerm.ddsp<-function(nw, arglist, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code for sp; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code for sp; valid types are:",paste(type.vec, collapse=","))
   dname<-"dsp"
   if(is.directed(nw)){
     conam <- paste("dsp",type,sep=".")
@@ -298,17 +298,17 @@ InitErgmTerm.ddsp<-function(nw, arglist, ...) {
 
 
 ################################################################################
-InitErgmTerm.dgwdsp<-function(nw, arglist, initialfit=FALSE, ...) {
+InitErgmTerm.dgwdsp<-function(nw, arglist, initialfit=FALSE, gw.cutoff=30, ...) {
   # the following line was commented out in <InitErgm.gwdsp>:
   #    ergm.checkdirected("gwdsp", is.directed(nw), requirement=FALSE)
   # so, I've not passed 'directed=FALSE' to <check.ErgmTerm>  
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("decay","fixed","cutoff","type", "alpha"),
                       vartypes = c("numeric","logical","numeric","character", "numeric"),
-                      defaultvalues = list(0, FALSE, 30,"OTP", NULL),
+                      defaultvalues = list(0, FALSE, gw.cutoff,"OTP", NULL),
                       required = c(FALSE, FALSE, FALSE, FALSE, FALSE))
   if(!is.null(a$alpha)){
-    stop("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".", call.=FALSE)
+    ergm_Init_abort("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".")
   }
   
   decay<-a$decay;fixed<-a$fixed
@@ -318,11 +318,11 @@ InitErgmTerm.dgwdsp<-function(nw, arglist, initialfit=FALSE, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code; valid types are:",paste(type.vec, collapse=","))
   dname<-"ddsp"
   
   if(!is.directed(nw)){  
-    stop("Use the gwnsp term for undirected networks.")
+    ergm_Init_abort("Use the gwnsp term for undirected networks.")
   }else{
     typecode<-which(type==type.vec)
     basenam<-paste("gwdsp",type,sep=".")
@@ -389,7 +389,7 @@ InitErgmTerm.dnsp<-function(nw, arglist, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code for sp; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code for sp; valid types are:",paste(type.vec, collapse=","))
   dname<-"nsp"
   if(is.directed(nw)){
     conam <- paste("nsp",type,sep=".")
@@ -418,17 +418,17 @@ InitErgmTerm.dnsp<-function(nw, arglist, ...) {
 
 
 ################################################################################
-InitErgmTerm.dgwnsp<-function(nw, arglist, initialfit=FALSE, ...) {
+InitErgmTerm.dgwnsp<-function(nw, arglist, initialfit=FALSE, gw.cutoff=30, ...) {
   # the following line was commented out in <InitErgm.gwnsp>:
   #    ergm.checkdirected("gwnsp", is.directed(nw), requirement=FALSE)
   # so, I've not passed 'directed=FALSE' to <check.ErgmTerm>  
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("decay","fixed","cutoff","type", "alpha"),
                       vartypes = c("numeric","logical","numeric","character", "numeric"),
-                      defaultvalues = list(0, FALSE, 30,"OTP", NULL),
+                      defaultvalues = list(0, FALSE, gw.cutoff,"OTP", NULL),
                       required = c(FALSE, FALSE, FALSE, FALSE, FALSE))
   if(!is.null(a$alpha)){
-    stop("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".", call.=FALSE)
+    ergm_Init_abort("For consistency with gw*degree terms, in all gw*sp and dgw*sp terms the argument ", sQuote("alpha"), " has been renamed to " ,sQuote("decay"), ".")
   }
   
   decay<-a$decay;fixed<-a$fixed
@@ -438,11 +438,11 @@ InitErgmTerm.dgwnsp<-function(nw, arglist, initialfit=FALSE, ...) {
   type<-toupper(a$type[1])
   type.vec<-c("OTP","ITP","RTP","OSP","ISP")
   if(!(type%in%type.vec))
-    stop("Illegal type code; valid types are:",paste(type.vec, collapse=","))
+    ergm_Init_abort("Illegal type code; valid types are:",paste(type.vec, collapse=","))
   dname<-"dnsp"
   
   if(!is.directed(nw)){  
-    stop("Use the gwnsp term for undirected networks.")
+    ergm_Init_abort("Use the gwnsp term for undirected networks.")
   }else{
     typecode<-which(type==type.vec)
     basenam<-paste("gwnsp",type,sep=".")
