@@ -1,11 +1,11 @@
 #  File R/control.san.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2003-2018 Statnet Commons
+#  Copyright 2003-2019 Statnet Commons
 #######################################################################
 
 
@@ -16,18 +16,32 @@
 #' 
 #' This function is only used within a call to the \code{\link{san}} function.
 #' See the \code{usage} section in \code{\link{san}} for details.
+#'
+#' @param SAN.maxit Number of temperature levels to use.
 #' 
-#' @param coef Vector of model coefficients used for MCMC simulations, one for
-#' each model term.
-#' @param SAN.tau Currently unused.
-#' @param SAN.invcov Initial inverse covariance matrix used to calculate
-#' Mahalanobis distance in determining how far a proposed MCMC move is from the
-#' \code{target.stats} vector.  If NULL, taken to be the covariance matrix
-#' returned when fitting the MPLE if \code{coef==NULL}, or the identity matrix
-#' otherwise.
-#' @param SAN.burnin Number of MCMC proposals before any sampling is done.
-#' @param SAN.interval Number of proposals between sampled statistics.
+#' @param SAN.tau Tuning parameter, specifying the temperature of the
+#'   process during the *penultimate* iteration. (During the last
+#'   iteration, the temperature is set to 0, resulting in a greedy
+#'   search, and during the previous iterations, the temperature is
+#'   set to `SAN.tau*(iterations left after this one)`.
+#' 
+#' @param SAN.invcov Initial inverse covariance matrix used to
+#'   calculate Mahalanobis distance in determining how far a proposed
+#'   MCMC move is from the \code{target.stats} vector.  If `NULL`,
+#'   initially set to the identity matrix, then during subsequent runs
+#'   estimated empirically.
+#'
+#' @param SAN.invcov.diag Whether to only use the diagonal of the
+#'   covariance matrix. It seems to work better in practice.
+#'
+#' @param SAN.nsteps.alloc Either a numeric vector or a function of
+#'   the number of runs giving a sequence of relative lengths of
+#'   simulated annealing runs.
+#'
+#' @param SAN.nsteps Number of MCMC proposals for all the annealing runs combined.
+#' @param SAN.samplesize Number of realisations' statistics to obtain for tuning purposes.
 #' @param SAN.init.maxedges Maximum number of edges expected in network.
+#' @param SAN.max.maxedges Hard upper bound on the number of edges in the network.
 #' @param SAN.prop.weights Specifies the method to allocate probabilities of
 #' being proposed to dyads. Defaults to \code{"default"}, which picks a
 #' reasonable default for the specified constraint.  Other possible values are
@@ -38,15 +52,6 @@
 #' @param SAN.packagenames Names of packages in which to look for change
 #' statistic functions in addition to those autodetected. This argument should
 #' not be needed outside of very strange setups.
-#' @param MPLE.max.dyad.types Maximum number of unique values of change
-#' statistic vectors, which are the predictors in a logistic regression used to
-#' calculate the MPLE.  This calculation uses a compression algorithm that
-#' allocates space based on \code{MPLE.max.dyad.types}
-#' @param MPLE.samplesize Not currently documented; used in
-#' conditional-on-degree version of MPLE.
-#' @param network.output R class with which to output networks. The options are
-#' "network" (default) and "edgelist.compressed" (which saves space but only
-#' supports networks without vertex attributes)
 #' @template term_options
 #' @template control_MCMC_parallel
 #' @template seed
@@ -54,23 +59,20 @@
 #' @seealso \code{\link{san}}
 #' @keywords models
 #' @export control.san
-control.san<-function(coef=NULL,
-
+control.san<-function(SAN.maxit=4,
                       SAN.tau=1,
                       SAN.invcov=NULL,
-                      SAN.burnin=100000,
-                      SAN.interval=10000,
+                      SAN.invcov.diag=FALSE,
+                      SAN.nsteps.alloc=function(nsim) 2^seq_len(nsim),
+                      SAN.nsteps=2^19,
+                      SAN.samplesize=2^12,
                       SAN.init.maxedges=20000,
+                      SAN.max.maxedges=2^26,
                       
                       SAN.prop.weights="default",
                       SAN.prop.args=list(),
                       SAN.packagenames=c(),
                       
-                      MPLE.max.dyad.types=1e6,
-                      MPLE.samplesize=50000,
-
-                      network.output="network",
-
                       term.options=list(),
 
                       seed=NULL,
