@@ -1,11 +1,11 @@
 #  File R/get.node.attr.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2003-2018 Statnet Commons
+#  Copyright 2003-2019 Statnet Commons
 #######################################################################
 ###############################################################################
 # The <get.node.attr> function returns the vector of covariates for the given
@@ -55,7 +55,7 @@
 #' 
 #' data(faux.mesa.high)
 #' get.node.attr(faux.mesa.high,'Grade')
-#' 
+#' @keywords internal
 #' @export get.node.attr
 get.node.attr <- function(nw, attrname, functionname=NULL, numeric=FALSE) {  
   ergm_get_vattr(attrname, nw, accept=if(numeric)"numeric"else"character")
@@ -64,32 +64,34 @@ get.node.attr <- function(nw, attrname, functionname=NULL, numeric=FALSE) {
 #' @name node-attr
 #' @title Specifying nodal attributes and their levels
 #'
-#' @description This document describes the ways in which to specify
-#'   nodal attribute or functions and which levels for categorical
-#'   factors to include. For the helper functions to facilitate this,
-#'   see [`node-attr-api`].
+#' @description This document describes the ways to specify nodal
+#'   attributes or functions of nodal attributes and which levels for
+#'   categorical factors to include. For the helper functions to
+#'   facilitate this, see [`node-attr-api`].
 #'
 #' @details
 #' 
-#' Term nodal attribute arguments, typically called `attrs`, `attrname`, `by`,
-#' `on`, etc. are interpreted as follows: \describe{
+#' Term nodal attribute arguments, typically called `attr`, `attrs`, `by`, or
+#' `on` are interpreted as follows: \describe{
 #' 
-#' \item{a single character string}{Extract the vertex attribute with
+#' \item{a character string}{Extract the vertex attribute with
 #' this name.}
 #' 
 #' \item{a character vector of length > 1}{Extract the vertex
-#' attributes and paste them together, separated by dots.}
+#' attributes and paste them together, separated by dots if the term
+#' expects categorical attributes and (typically) combine into a
+#' covariate matrix if it expects quantitative attributes.}
 #' 
 #' \item{a function}{The function is called on the LHS network,
-#' expected to return a vector of appropriate length. (Shorter vectors
-#' will be recycled as needed.)}
+#' expected to return a vector or matrix of appropriate
+#' dimension. (Shorter vectors and matrix columns will be recycled as needed.)}
 #' 
 #' \item{a formula}{The expression on the RHS of the formula is
 #' evaluated in an environment of the vertex attributes of the
-#' network, expected to return a vector of appropriate
-#' length. (Shorter vectors will be recycled as needed.) Within this
-#' expression, the network itself accessible as either `.` or
-#' `.nw`. For example,
+#' network, expected to return a vector or matrix of appropriate
+#' dimension. (Shorter vectors and matrix columns will be recycled as
+#' needed.) Within this expression, the network itself accessible as
+#' either `.` or `.nw`. For example,
 #' `nodecov(~abs(Grade-mean(Grade))/network.size(.))` would return the
 #' absolute difference of each actor's "Grade" attribute from its
 #' network-wide mean, divided by the network size.}
@@ -97,17 +99,23 @@ get.node.attr <- function(nw, attrname, functionname=NULL, numeric=FALSE) {
 #' }
 #'
 #' For categorical attributes, to select which levels are of interest
-#' and their ordering, use the argument `levels`.  It is interpreted
+#' and their ordering, use the argument `levels`.  Selection of nodes (from
+#' the appropriate vector of nodal indices) is likewise handled as the
+#' selection of levels, using the argument `nodes`.  These arguments are interpreted
 #' as follows: \describe{
 #'
 #' \item{an expression wrapped in [I()]}{Use the given list of levels
 #' as is.}
 #' 
-#' \item{a numeric or logical vector}{Used for indexing of the default
-#' set of levels (typically, unique values of the attribute) in
+#' \item{a numeric or logical vector}{Used for indexing of a list of
+#' all possible levels (typically, unique values of the attribute) in
 #' default older (typically lexicographic), i.e.,
-#' `sort(unique(attr))[levels]`. Negative values exclude. To specify
-#' numeric or logical levels literally, wrap in [I()].}
+#' `sort(unique(attr))[levels]`. In particular, `levels=TRUE` will
+#' retain all levels. Negative values exclude. To specify numeric or
+#' logical levels literally, wrap in [I()].}
+#'
+#'\item{[`NULL`]}{Retain all possible levels; usually equivalent to
+#' passing `TRUE`.}
 #'
 #' \item{a character vector}{Use as is.}
 #' 
@@ -124,13 +132,19 @@ get.node.attr <- function(nw, attrname, functionname=NULL, numeric=FALSE) {
 #' 
 #' }
 #' 
-#' Note that `levels` often has a default that is sensible for the
+#' Note that `levels` or `nodes` often has a default that is sensible for the
 #' term in question.
 #' 
 #' @aliases attrname on by attrs
 #' @examples
 #'
 #' data(faux.mesa.high)
+#' 
+#' # Activity by grade with a baseline grade excluded:
+#' summary(faux.mesa.high~nodefactor(~Grade))
+#' # Retain all levels:
+#' summary(faux.mesa.high~nodefactor(~Grade, levels=TRUE)) # or levels=NULL
+#' 
 #' # Mixing between lower and upper grades:
 #' summary(faux.mesa.high~mm(~Grade>=10))
 #' # Mixing between grades 7 and 8 only:
@@ -163,6 +177,8 @@ NULL
 #'   vector returned or the length permited: `"n"` for full network,
 #'   `"b1"` for first mode of a bipartite network, and `"b2"` for the
 #'   second.
+#' @param multiple Handling of multiple attributes or matrix or data
+#'   frame output. See the Details section for the specification.
 #' @param accept A character vector listing permitted data types for
 #'   the output. See the Details section for the specification.
 #' @param ... Additional argument to the functions of network or to
@@ -194,9 +210,23 @@ NULL
 #' \item{`"positive"`}{Accept a strictly positive number or logical.}
 #' }
 #'
+#' \describe{
+#' 
+#' \item{`"paste"`}{Paste together with dot as the separator.}
+#' 
+#' \item{`"stop"`}{Fail with an error message.}
+#'
+#' \item{`"matrix"`}{Construct and/or return a matrix whose rows correspond to vertices.}
+#'
+#' }
+#'
 #'
 NULL
 
+#' @rdname node-attr-api
+#' @export
+ERGM_GET_VATTR_MULTIPLE_TYPES <- c("paste", "matrix", "stop")
+
 #' @rdname node-attr-api
 #'
 #' @description `ergm_get_vattr` extracts and processes the specified
@@ -217,22 +247,35 @@ NULL
 #' ergm_get_vattr(c("wealth","priorates"), flomarriage)
 #' ergm_get_vattr(~priorates>30, flomarriage)
 #' (a <- ergm_get_vattr(~cut(priorates,c(-Inf,0,20,40,60,Inf),label=FALSE)-1, flomarriage))
+#' @keywords internal
 #' @export
-ergm_get_vattr <- function(object, nw, accept="character", bip=c("n","b1","b2"), ...){
+ergm_get_vattr <- function(object, nw, accept="character", bip=c("n","b1","b2"), multiple=if(accept=="character") "paste" else "stop", ...){
   bip <- match.arg(bip)
+  multiple <- match.arg(multiple, ERGM_GET_VATTR_MULTIPLE_TYPES)
   UseMethod("ergm_get_vattr")
 }
 
+.handle_multiple <- function(a, multiple){
+  if(!is.list(a)) a <- list(a)
+  a <- do.call(cbind, a)
+  if(ncol(a)>1)
+    switch(multiple,
+           paste =  apply(a, 1, paste, collapse="."),
+           matrix = a,
+           stop = ergm_Init_abort("This term does not accept multiple vertex attributes or matrix vertex attribute functions."))
+  else c(a)
+}
+
 .rightsize_vattr <- function(a, nw, bip){
   rep_len_warn <- function(x, length.out){
-    if(length.out%%length(x)) ergm_Init_warn("Length of vertex attribute vector is not a multiple of network size or bipartite group size.")
-    rep_len(x, length.out)
+    if(length.out%%NVL(nrow(x), length(x))) ergm_Init_warn("Network size or bipartite group size is not a multiple of the length of vertex attributes.")
+    if(is.null(nrow(x))) rep_len(x, length.out) else apply(x, 2, rep_len, length.out)
   }
   if(!is.bipartite(nw) || bip=="n") rep_len_warn(a, network.size(nw))
-  else if(length(a)==network.size(nw)) # Input vector is n-long, need to trim.
+  else if(NVL(nrow(a), length(a))==network.size(nw)) # Input vector is n-long, need to trim.
     switch(bip,
-           b1 = a[seq_len(nw%n%"bipartite")],
-           b2 = a[-seq_len(nw%n%"bipartite")])
+           b1 = if(is.null(nrow(a))) a[seq_len(nw%n%"bipartite")] else a[seq_len(nw%n%"bipartite"),,drop=FALSE],
+           b2 = if(is.null(nrow(a))) a[-seq_len(nw%n%"bipartite")] else a[-seq_len(nw%n%"bipartite"),,drop=FALSE])
   else # Othewise, recycle until the right length.
     rep_len_warn(a, switch(bip,
                            b1=nw%n%"bipartite",
@@ -268,16 +311,18 @@ ergm_get_vattr <- function(object, nw, accept="character", bip=c("n","b1","b2"),
 
 #' @rdname node-attr-api
 #' @importFrom purrr "%>%" "map" "pmap_chr"
-#' @importFrom rlang set_attrs
+#' @importFrom rlang set_attrs set_names
 #' @export
-ergm_get_vattr.character <- function(object, nw, accept="character", bip=c("n","b1","b2"), ...){
+ergm_get_vattr.character <- function(object, nw, accept="character", bip=c("n","b1","b2"), multiple=if(accept=="character") "paste" else "stop", ...){
+  bip <- match.arg(bip)
+  multiple <- match.arg(multiple, ERGM_GET_VATTR_MULTIPLE_TYPES)
+
   missing_attr <- setdiff(object, list.vertex.attributes(nw))
   if(length(missing_attr)){
     ergm_Init_abort(paste.and(sQuote(missing_attr)), " is/are not valid nodal attribute(s).")
   }
 
-  (if(length(object)==1) nw%v%object
-   else object %>% map(~nw%v%.) %>% pmap_chr(paste, sep=".")) %>%
+  object %>% map(~nw%v%.) %>% set_names(object) %>% .handle_multiple(multiple=multiple) %>%
     .rightsize_vattr(nw, bip) %>% set_attrs(name=paste(object, collapse=".")) %>%
     .check_acceptable(accept=accept, xspec=object)
 }
@@ -285,9 +330,13 @@ ergm_get_vattr.character <- function(object, nw, accept="character", bip=c("n","
 
 #' @rdname node-attr-api
 #' @export
-ergm_get_vattr.function <- function(object, nw, accept="character", bip=c("n","b1","b2"), ...){
+ergm_get_vattr.function <- function(object, nw, accept="character", bip=c("n","b1","b2"), multiple=if(accept=="character") "paste" else "stop", ...){
+  bip <- match.arg(bip)
+  multiple <- match.arg(multiple, ERGM_GET_VATTR_MULTIPLE_TYPES)
+
   ERRVL(try(object(nw, ...) %>%
-            .rightsize_vattr(nw, bip),
+            .rightsize_vattr(nw, bip) %>% .handle_multiple(multiple=multiple) %>%
+            set_attrs(name=strtrim(despace(paste(deparse(body(object)),collapse="\n")),80)),
             silent=TRUE),
         ergm_Init_abort(.)) %>%
     .check_acceptable(accept=accept)
@@ -298,15 +347,18 @@ ergm_get_vattr.function <- function(object, nw, accept="character", bip=c("n","b
 #' @importFrom purrr "%>%" map set_names when
 #' @importFrom tibble lst
 #' @export
-ergm_get_vattr.formula <- function(object, nw, accept="character", bip=c("n","b1","b2"), ...){
+ergm_get_vattr.formula <- function(object, nw, accept="character", bip=c("n","b1","b2"), multiple=if(accept=="character") "paste" else "stop", ...){
+  bip <- match.arg(bip)
+  multiple <- match.arg(multiple, ERGM_GET_VATTR_MULTIPLE_TYPES)
+
   a <- list.vertex.attributes(nw)
   vlist <- c(a %>% map(~nw%v%.) %>% set_names(a),
              lst(`.`=nw, .nw=nw, ...))
 
-  e <- object[[length(object)]]
+  e <- ult(object)
   ERRVL(try({
     eval(e, envir=vlist, enclos=environment(object)) %>%
-      .rightsize_vattr(nw, bip) %>%
+      .rightsize_vattr(nw, bip) %>% .handle_multiple(multiple=multiple) %>%
       set_attrs(name=if(length(object)>2) eval_lhs.formula(object) else despace(paste(deparse(e),collapse="\n")))
   }, silent=TRUE),
   ergm_Init_abort(.)) %>%
@@ -373,7 +425,7 @@ ergm_attr_levels.function <- function(object, attr, nw, levels=sort(unique(attr)
 #' @export
 ergm_attr_levels.formula <- function(object, attr, nw, levels=sort(unique(attr)), ...){
   vlist <- lst(`.`=levels, .levels=levels, .attr=attr, .nw=nw, ...)
-  e <- object[[length(object)]]
+  e <- ult(object)
   object <- eval(e, envir=vlist, enclos=environment(object))  
   ergm_attr_levels(object, attr, nw, levels, ...)
 }
