@@ -1,11 +1,11 @@
 /*  File src/CD.c in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #include "CD.h"
 
@@ -22,12 +22,12 @@
 
  and don't forget that tail -> head
 *****************/
-void CD_wrapper(int *dnumnets, int *nedges,
+void CD_wrapper(int *nedges,
 		  int *tails, int *heads,
 		  int *dn, int *dflag, int *bipartite, 
 		  int *nterms, char **funnames,
 		  char **sonames, 
-		  char **MHproposaltype, char **MHproposalpackage,
+		  char **MHProposaltype, char **MHProposalpackage,
 		double *inputs, double *theta0, int *samplesize, int *CDparams,
 		  double *sample,
 		  int *fVerbose, 
@@ -37,12 +37,11 @@ void CD_wrapper(int *dnumnets, int *nedges,
   int directed_flag;
   Vertex n_nodes, bip, *undotail, *undohead;
   /* Edge n_networks; */
-  Network nw[1];
+  Network *nwp;
   Model *m;
-  MHproposal MH;
+  MHProposal *MHp;
   
   n_nodes = (Vertex)*dn; 
-  /* n_networks = (Edge)*dnumnets;  */
   bip = (Vertex)*bipartite; 
   
   GetRNGstate();  /* R function enabling uniform RNG */
@@ -52,31 +51,31 @@ void CD_wrapper(int *dnumnets, int *nedges,
   m=ModelInitialize(*funnames, *sonames, &inputs, *nterms);
 
   /* Form the network */
-  nw[0]=NetworkInitialize(tails, heads, nedges[0], 
+  nwp=NetworkInitialize((Vertex*)tails, (Vertex*)heads, nedges[0], 
                           n_nodes, directed_flag, bip, 0, 0, NULL);
   
-  MH_init(&MH,
-	  *MHproposaltype, *MHproposalpackage,
+  MHp=MHProposalInitialize(
+	  *MHProposaltype, *MHProposalpackage,
 	  inputs,
 	  *fVerbose,
-	  nw, attribs, maxout, maxin, minout, minin,
+	  nwp, attribs, maxout, maxin, minout, minin,
 	  *condAllDegExact, *attriblength);
 
-  undotail = calloc(MH.ntoggles * CDparams[0] * CDparams[1], sizeof(Vertex));
-  undohead = calloc(MH.ntoggles * CDparams[0] * CDparams[1], sizeof(Vertex));
-  double *extraworkspace = calloc(m->n_stats, sizeof(double));
+  undotail = Calloc(MHp->ntoggles * CDparams[0] * CDparams[1], Vertex);
+  undohead = Calloc(MHp->ntoggles * CDparams[0] * CDparams[1], Vertex);
+  double *extraworkspace = Calloc(m->n_stats, double);
 
-  *status = CDSample(&MH,
+  *status = CDSample(MHp,
 		     theta0, sample, *samplesize, CDparams, undotail, undohead,
-		     *fVerbose, nw, m, extraworkspace);
+		     *fVerbose, nwp, m, extraworkspace);
   
-  free(undotail);
-  free(undohead);
-  free(extraworkspace);
-  MH_free(&MH);
+  Free(undotail);
+  Free(undohead);
+  Free(extraworkspace);
+  MHProposalDestroy(MHp);
 
   ModelDestroy(m);
-  NetworkDestroy(nw);
+  NetworkDestroy(nwp);
   PutRNGstate();  /* Disable RNG before returning */
 }
 
@@ -91,7 +90,7 @@ void CD_wrapper(int *dnumnets, int *nedges,
  networks in the sample.  Put all the sampled statistics into
  the networkstatistics array. 
 *********************/
-MCMCStatus CDSample(MHproposal *MHp,
+MCMCStatus CDSample(MHProposal *MHp,
 		    double *theta, double *networkstatistics, 
 		    int samplesize, int *CDparams, Vertex *undotail, Vertex *undohead, int fVerbose,
 		    Network *nwp, Model *m, double *extraworkspace){
@@ -153,7 +152,7 @@ MCMCStatus CDSample(MHproposal *MHp,
  the networkstatistics vector.  In other words, this function 
  essentially generates a sample of size one
 *********************/
-MCMCStatus CDStep(MHproposal *MHp,
+MCMCStatus CDStep(MHProposal *MHp,
 		  double *theta, double *networkstatistics,
 		  int *CDparams, int *staken,
 		  Vertex *undotail, Vertex *undohead,
@@ -178,14 +177,14 @@ MCMCStatus CDStep(MHproposal *MHp,
 	  error("Something very bad happened during proposal. Memory has not been deallocated, so restart R soon.");
 	  
 	case MH_IMPOSSIBLE:
-	  Rprintf("MH Proposal function encountered a configuration from which no toggle(s) can be proposed.\n");
+	  Rprintf("MH MHProposal function encountered a configuration from which no toggle(s) can be proposed.\n");
 	  return MCMC_MH_FAILED;
 	  
 	case MH_UNSUCCESSFUL:
-	  warning("MH Proposal function failed to find a valid proposal.");
+	  warning("MH MHProposal function failed to find a valid proposal.");
 	  unsuccessful++;
 	  if(unsuccessful>*staken*MH_QUIT_UNSUCCESSFUL){
-	    Rprintf("Too many MH Proposal function failures.\n");
+	    Rprintf("Too many MH MHProposal function failures.\n");
 	    return MCMC_MH_FAILED;
 	  }
 	  continue;
@@ -197,7 +196,7 @@ MCMCStatus CDStep(MHproposal *MHp,
       }
       
       if(fVerbose>=5){
-	Rprintf("Proposal: ");
+	Rprintf("MHproposal: ");
 	for(unsigned int i=0; i<MHp->ntoggles; i++)
 	  Rprintf(" (%d, %d)", MHp->toggletail[i], MHp->togglehead[i]);
 	Rprintf("\n");
