@@ -1,11 +1,11 @@
 /*  File src/MCMC.c in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #include "MCMC.h"
 
@@ -22,12 +22,12 @@
 
  and don't forget that tail -> head
 *****************/
-void MCMC_wrapper(int *dnumnets, int *nedges,
+void MCMC_wrapper(int *nedges,
 		  int *tails, int *heads,
 		  int *dn, int *dflag, int *bipartite, 
 		  int *nterms, char **funnames,
 		  char **sonames, 
-		  char **MHproposaltype, char **MHproposalpackage,
+		  char **MHProposaltype, char **MHProposalpackage,
 		  double *inputs, double *theta0, int *samplesize, 
 		  double *sample, int *burnin, int *interval,  
 		  int *newnetworktails, 
@@ -40,12 +40,11 @@ void MCMC_wrapper(int *dnumnets, int *nedges,
   int directed_flag;
   Vertex n_nodes, nmax, bip;
   /* Edge n_networks; */
-  Network nw[1];
+  Network *nwp;
   Model *m;
-  MHproposal MH;
+  MHProposal *MHp;
   
   n_nodes = (Vertex)*dn; 
-  /* n_networks = (Edge)*dnumnets;  */
   nmax = (Edge)abs(*maxedges);
   bip = (Vertex)*bipartite; 
   
@@ -56,31 +55,31 @@ void MCMC_wrapper(int *dnumnets, int *nedges,
   m=ModelInitialize(*funnames, *sonames, &inputs, *nterms);
 
   /* Form the network */
-  nw[0]=NetworkInitialize(tails, heads, nedges[0], 
+  nwp=NetworkInitialize((Vertex*)tails, (Vertex*)heads, nedges[0], 
                           n_nodes, directed_flag, bip, 0, 0, NULL);
   
-  MH_init(&MH,
-	  *MHproposaltype, *MHproposalpackage,
+  MHp=MHProposalInitialize(
+	  *MHProposaltype, *MHProposalpackage,
 	  inputs,
 	  *fVerbose,
-	  nw, attribs, maxout, maxin, minout, minin,
+	  nwp, attribs, maxout, maxin, minout, minin,
 	  *condAllDegExact, *attriblength);
 
-  *status = MCMCSample(&MH,
+  *status = MCMCSample(MHp,
 		       theta0, sample, *samplesize,
 		       *burnin, *interval,
-		       *fVerbose, nmax, nw, m);
+		       *fVerbose, nmax, nwp, m);
   
-  MH_free(&MH);
+  MHProposalDestroy(MHp);
         
-/* Rprintf("Back! %d %d\n",nw[0].nedges, nmax); */
+/* Rprintf("Back! %d %d\n",nwp[0].nedges, nmax); */
 
   /* record new generated network to pass back to R */
   if(*status == MCMC_OK && *maxedges>0 && newnetworktails && newnetworkheads)
-    newnetworktails[0]=newnetworkheads[0]=EdgeTree2EdgeList(newnetworktails+1,newnetworkheads+1,nw,nmax-1);
+    newnetworktails[0]=newnetworkheads[0]=EdgeTree2EdgeList((Vertex*)newnetworktails+1,(Vertex*)newnetworkheads+1,nwp,nmax-1);
   
   ModelDestroy(m);
-  NetworkDestroy(nw);
+  NetworkDestroy(nwp);
   PutRNGstate();  /* Disable RNG before returning */
 }
 
@@ -95,7 +94,7 @@ void MCMC_wrapper(int *dnumnets, int *nedges,
  networks in the sample.  Put all the sampled statistics into
  the networkstatistics array. 
 *********************/
-MCMCStatus MCMCSample(MHproposal *MHp,
+MCMCStatus MCMCSample(MHProposal *MHp,
 		double *theta, double *networkstatistics, 
 		int samplesize, int burnin, 
 		int interval, int fVerbose, int nmax,
@@ -125,7 +124,7 @@ MCMCStatus MCMCSample(MHproposal *MHp,
   if(MetropolisHastings(MHp, theta, networkstatistics, burnin, &staken,
 			fVerbose, nwp, m)!=MCMC_OK)
     return MCMC_MH_FAILED;
-  if(nmax!=0 && nwp->nedges >= nmax-1){
+  if(nmax!=0 && EDGECOUNT(nwp) >= nmax-1){
     return MCMC_TOO_MANY_EDGES;
   }
   
@@ -150,7 +149,7 @@ MCMCStatus MCMCSample(MHproposal *MHp,
       if(MetropolisHastings(MHp, theta, networkstatistics, interval, &staken,
 			    fVerbose, nwp, m)!=MCMC_OK)
 	return MCMC_MH_FAILED;
-      if(nmax!=0 && nwp->nedges >= nmax-1){
+      if(nmax!=0 && EDGECOUNT(nwp) >= nmax-1){
 	return MCMC_TOO_MANY_EDGES;
       }
       tottaken += staken;
@@ -196,7 +195,7 @@ MCMCStatus MCMCSample(MHproposal *MHp,
  the networkstatistics vector.  In other words, this function 
  essentially generates a sample of size one
 *********************/
-MCMCStatus MetropolisHastings(MHproposal *MHp,
+MCMCStatus MetropolisHastings(MHProposal *MHp,
 			      double *theta, double *networkstatistics,
 			      int nsteps, int *staken,
 			      int fVerbose,
@@ -216,14 +215,14 @@ MCMCStatus MetropolisHastings(MHproposal *MHp,
 	error("Something very bad happened during proposal. Memory has not been deallocated, so restart R soon.");
 	
       case MH_IMPOSSIBLE:
-	Rprintf("MH Proposal function encountered a configuration from which no toggle(s) can be proposed.\n");
+	Rprintf("MH MHProposal function encountered a configuration from which no toggle(s) can be proposed.\n");
 	return MCMC_MH_FAILED;
 	
       case MH_UNSUCCESSFUL:
-	warning("MH Proposal function failed to find a valid proposal.");
+	warning("MH MHProposal function failed to find a valid proposal.");
 	unsuccessful++;
 	if(unsuccessful>taken*MH_QUIT_UNSUCCESSFUL){
-	  Rprintf("Too many MH Proposal function failures.\n");
+	  Rprintf("Too many MH MHProposal function failures.\n");
 	  return MCMC_MH_FAILED;
 	}
       case MH_CONSTRAINT:
@@ -232,7 +231,7 @@ MCMCStatus MetropolisHastings(MHproposal *MHp,
     }
     
     if(fVerbose>=5){
-      Rprintf("Proposal: ");
+      Rprintf("MHproposal: ");
       for(unsigned int i=0; i<MHp->ntoggles; i++)
 	Rprintf(" (%d, %d)", MHp->toggletail[i], MHp->togglehead[i]);
       Rprintf("\n");
@@ -300,7 +299,7 @@ void MCMCPhase12 (int *tails, int *heads, int *dnedges,
 		  int *dn, int *dflag, int *bipartite, 
 		  int *nterms, char **funnames,
 		  char **sonames, 
-		  char **MHproposaltype, char **MHproposalpackage,
+		  char **MHProposaltype, char **MHProposalpackage,
 		  double *inputs, 
 		  double *theta0, int *samplesize,
 		  double *gain, double *meanstats, int *phase1, int *nsub,
@@ -316,9 +315,9 @@ void MCMCPhase12 (int *tails, int *heads, int *dnedges,
   int nphase1, nsubphases;
   Vertex n_nodes, bip;
   Edge n_edges, nmax;
-  Network nw[2];
+  Network *nwp;
   Model *m;
-  MHproposal MH;
+  MHProposal *MHp;
   
   nphase1 = *phase1; 
   nsubphases = *nsub;
@@ -335,31 +334,31 @@ void MCMCPhase12 (int *tails, int *heads, int *dnedges,
   m=ModelInitialize(*funnames, *sonames, &inputs, *nterms);
 
   /* Form the missing network */
-  nw[0]=NetworkInitialize(tails, heads, n_edges,
+  nwp=NetworkInitialize((Vertex*)tails, (Vertex*)heads, n_edges,
                           n_nodes, directed_flag, bip, 0, 0, NULL);
 
 
-  MH_init(&MH,
-	  *MHproposaltype, *MHproposalpackage,
+  MHp=MHProposalInitialize(
+	  *MHProposaltype, *MHProposalpackage,
 	  inputs,
 	  *fVerbose,
-	  nw, attribs, maxout, maxin, minout, minin,
+	  nwp, attribs, maxout, maxin, minout, minin,
 	  *condAllDegExact, *attriblength);
   
-  MCMCSamplePhase12 (&MH,
+  MCMCSamplePhase12 (MHp,
 		     theta0, *gain, meanstats, nphase1, nsubphases, sample, *samplesize,
 		     *burnin, *interval,
-		     (int)*fVerbose, nw, m);
+		     (int)*fVerbose, nwp, m);
 
-  MH_free(&MH);
+  MHProposalDestroy(MHp);
   
   /* record new generated network to pass back to R */
   if(nmax>0 && newnetworktails && newnetworkheads)
-    newnetworktails[0]=newnetworkheads[0]=EdgeTree2EdgeList(newnetworktails+1,newnetworkheads+1,nw,nmax-1);
+    newnetworktails[0]=newnetworkheads[0]=EdgeTree2EdgeList((Vertex*)newnetworktails+1,(Vertex*)newnetworkheads+1,nwp,nmax-1);
 
   ModelDestroy(m);
 
-  NetworkDestroy(nw);
+  NetworkDestroy(nwp);
   PutRNGstate();  /* Disable RNG before returning */
 }
 
@@ -373,7 +372,7 @@ void MCMCPhase12 (int *tails, int *heads, int *dnedges,
  networks in the sample.  Put all the sampled statistics into
  the networkstatistics array. 
 *********************/
-void MCMCSamplePhase12(MHproposal *MHp,
+void MCMCSamplePhase12(MHProposal *MHp,
 		       double *theta, double gain, double *meanstats, int nphase1, int nsubphases, double *networkstatistics, 
 		       int samplesize, int burnin, 
 		       int interval, int fVerbose,
@@ -395,9 +394,9 @@ void MCMCSamplePhase12(MHproposal *MHp,
   all be zero
   *********************/
   double *ubar, *u2bar, *aDdiaginv;
-  ubar = (double *)malloc( m->n_stats * sizeof(double));
-  u2bar = (double *)malloc( m->n_stats * sizeof(double));
-  aDdiaginv = (double *)malloc( m->n_stats * sizeof(double));
+  ubar = (double *)Calloc(m->n_stats, double);
+  u2bar = (double *)Calloc(m->n_stats, double);
+  aDdiaginv = (double *)Calloc(m->n_stats, double);
   for (j=0; j < m->n_stats; j++){
     networkstatistics[j] = -meanstats[j];
     ubar[j] = 0.0;
@@ -535,7 +534,7 @@ void MCMCSamplePhase12(MHproposal *MHp,
     Rprintf("Phase 3: MCMC-Newton-Raphson\n");
   }
 
-  free(ubar);
-  free(u2bar);
+  Free(ubar);
+  Free(u2bar);
 }
 
