@@ -1,18 +1,18 @@
 /*  File src/MPLE.h in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #ifndef MPLE_H
 #define MPLE_H
 
-#include "edgetree.h"
-#include "changestat.h"
-#include "model.h"
+#include "ergm_edgetree.h"
+#include "ergm_changestat.h"
+#include "ergm_model.h"
 #include "ergm_rlebdm.h"
 
 void MPLE_wrapper(int *tails, int *heads, int *dnedges,
