@@ -1,25 +1,25 @@
 /*  File src/allstatistics.c in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #include "MPLE.h"
-#include "changestat.h"
+#include "ergm_changestat.h"
 
-void RecurseOffOn(int *nodelist1,int *nodelist2, int nodelistlength, 
-       int currentnodes, double *changeStats, double *cumulativeStats,
-       double *covmat, int *weightsvector,
-       int maxNumDyadTypes, Network *nwp, Model *m);
+void RecurseOffOn(Vertex *nodelist1,Vertex *nodelist2, Vertex nodelistlength, 
+       Vertex currentnodes, double *changeStats, double *cumulativeStats,
+       double *covmat, unsigned int *weightsvector,
+       unsigned int maxNumDyadTypes, Network *nwp, Model *m);
 
-unsigned int InsNetStatRow(double *newRow, double *matrix, int rowLength, 
-       int numRows, int *weights );
+unsigned int InsNetStatRow(double *newRow, double *matrix, unsigned int rowLength, 
+       unsigned int numRows, unsigned int *weights );
 
-unsigned int hashNetStatRow(double *newRow, int rowLength, 
-       int numRows);
+unsigned int hashNetStatRow(double *newRow, unsigned int rowLength, 
+       unsigned int numRows);
 
         
 /* *****************
@@ -46,20 +46,19 @@ void AllStatistics (
        double *covmat,
 		   int *weightsvector,
        int *maxNumDyadTypes) {
-  Network nw, *nwp;
+  Network *nwp;
 
   Vertex n_nodes = (Vertex) *dn; 
-  int directed_flag = *dflag;
-  int nodelistlength, rowmax, *nodelist1, *nodelist2;
+  unsigned int directed_flag = *dflag;
+  Vertex nodelistlength, rowmax, *nodelist1, *nodelist2;
   Vertex bip = (Vertex) *bipartite;
   Model *m;
   ModelTerm *mtp;
 
   /* Step 1:  Initialize empty network and initialize model */
   GetRNGstate(); /* Necessary for R random number generator */
-  nw=NetworkInitialize(tails, heads, *dnedges,
+  nwp=NetworkInitialize((Vertex*)tails, (Vertex*)heads, *dnedges,
 		       n_nodes, directed_flag, bip, 0, 0, NULL);
-  nwp = &nw;
   m=ModelInitialize(*funnames, *sonames, &inputs, *nterms);
   
   /* Step 2:  Build nodelist1 and nodelist2, which together give all of the
@@ -71,8 +70,8 @@ void AllStatistics (
     nodelistlength = N_NODES * (N_NODES-1) / (DIRECTED? 1 : 2);
     rowmax = N_NODES;
   }
-  nodelist1 = (int *) R_alloc(nodelistlength, sizeof(int));
-  nodelist2 = (int *) R_alloc(nodelistlength, sizeof(int));
+  nodelist1 = (Vertex *) R_alloc(nodelistlength, sizeof(int));
+  nodelist2 = (Vertex *) R_alloc(nodelistlength, sizeof(int));
   int count = 0;
   for(int i=1; i < rowmax; i++) {
     for(int j = MAX(i,BIPARTITE)+1; j <= N_NODES; j++) {
@@ -103,7 +102,7 @@ void AllStatistics (
 
   /* Step 4:  Begin recursion */
   RecurseOffOn(nodelist1, nodelist2, nodelistlength, 0, changeStats, 
-           cumulativeStats, covmat, weightsvector, *maxNumDyadTypes, nwp, m);
+	       cumulativeStats, covmat, (unsigned int*) weightsvector, *maxNumDyadTypes, nwp, m);
 
   /* Step 5:  Deallocate memory and return */
   ModelDestroy(m);
@@ -112,15 +111,15 @@ void AllStatistics (
 }
 
 void RecurseOffOn(
-       int *nodelist1, 
-       int *nodelist2, 
-       int nodelistlength, 
-       int currentnodes, 
+       Vertex *nodelist1, 
+       Vertex *nodelist2, 
+       Vertex nodelistlength, 
+       Vertex currentnodes, 
        double *changeStats,
        double *cumulativeStats,
        double *covmat,
-       int *weightsvector,
-		   int maxNumDyadTypes, 
+       unsigned int *weightsvector,
+       unsigned int maxNumDyadTypes, 
        Network *nwp, 
        Model *m) {
   ModelTerm *mtp;
@@ -142,7 +141,7 @@ void RecurseOffOn(
     /* Calculate the change statistic(s) associated with toggling the 
        dyad represented by nodelist1[currentnodes], nodelist2[currentnodes] */
     for (mtp=m->termarray; mtp < m->termarray + m->n_terms; mtp++){
-      (*(mtp->d_func))(1, nodelist1+currentnodes, nodelist2+currentnodes, mtp, nwp);
+      (*(mtp->d_func))(1, (Vertex*)nodelist1+currentnodes, (Vertex*)nodelist2+currentnodes, mtp, nwp);
     }
     for (int j=0; j < m->n_stats; j++) cumulativeStats[j] += changeStats[j];
     /* Now toggle the dyad so it's ready for the next pass */
@@ -153,9 +152,9 @@ void RecurseOffOn(
 unsigned int InsNetStatRow(
                      double *newRow, 
                      double *matrix, 
-                     int rowLength, 
-                     int numRows, 
-                     int *weights ){
+                     unsigned int rowLength, 
+                     unsigned int numRows, 
+                     unsigned int *weights ){
   unsigned int pos, round;
   unsigned int hash_pos = hashNetStatRow(newRow, rowLength, numRows);
   
@@ -181,7 +180,7 @@ Uses Jenkins One-at-a-Time hash.
 
 numRows should, ideally, be a power of 2, but it doesn't have to be.
 **************/
-unsigned int hashNetStatRow(double *newRow, int rowLength, int numRows) {
+unsigned int hashNetStatRow(double *newRow, unsigned int rowLength, unsigned int numRows) {
   /* Cast all pointers to unsigned char pointers, since data need to 
      be fed to the hash function one byte at a time. */
   unsigned char *cnewRow = (unsigned char *) newRow;
