@@ -1,17 +1,17 @@
 /*  File src/changestats_transitiveties.h in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #ifndef CHANGESTATS_H
 #define CHANGESTATS_H
 
-#include "edgetree.h"
-#include "changestat.h"
+#include "ergm_edgetree.h"
+#include "ergm_changestat.h"
 
 D_CHANGESTAT_FN(d_cyclicalties); S_CHANGESTAT_FN(s_cyclicalties);
 D_CHANGESTAT_FN(d_transitiveties2); S_CHANGESTAT_FN(s_transitiveties2);
