@@ -1,16 +1,16 @@
 /*  File src/geodist.h in package ergm, part of the Statnet suite
- *  of packages for network analysis, http://statnet.org .
+ *  of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  http://statnet.org/attribution
+ *  https://statnet.org/attribution
  *
- *  Copyright 2003-2018 Statnet Commons
+ *  Copyright 2003-2019 Statnet Commons
  */
 #ifndef GEODIST_H
 #define GEODIST_H
 
-#include "edgetree.h"
+#include "ergm_edgetree.h"
 
 void node_geodesics (int *edgelist, int *nnodes, int *nodelist,
                      int *nedges, int *nodecolor, int *dist, 
