@@ -1,11 +1,11 @@
 #  File tests/target_offset.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2003-2018 Statnet Commons
+#  Copyright 2003-2019 Statnet Commons
 #######################################################################
 library(ergm)
 
@@ -25,9 +25,9 @@ if(!is.null(out)){
   mcmc.diagnostics(out)
 }
 
-set.seed(11)
+set.seed(1)
 
-out <- tryCatch(ergm(flomarriage~offset(edges)+edges+gwdegree(fix=FALSE)+degree(0)+offset(degree(1)),target.stats=summary(flomarriage~edges+gwdegree(fix=FALSE)+degree(0)),
+out <- tryCatch(ergm(flomarriage~offset(edges)+edges+gwdegree(0, fix=FALSE)+degree(0)+offset(degree(1)),target.stats=summary(flomarriage~edges+gwdegree(0, fix=FALSE)+degree(0)),
               offset.coef=c(0,-0.25)),
          error=function(e) stop('error in target + offset test', e), 
          warning=warnf)
